<?php

namespace Drupal\gitinfo\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;

/**
 * Class GitinfoSettingsForm.
 */
class GitinfoSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'gitinfo.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'gitinfo_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $disabled = FALSE;
    $explain_title = NULL;
    $description = $this->t('For example &#039;/var/www/html&#039; to monitor all projects in that folder. One directory per line. The <strong>absolute path</strong> from system root must be used, starting with a slash ( / ).<br />Projects running in a Virtual Machine can be added, as long they contain folders that are under Git version control.');
    $status_report = Link::fromTextAndUrl($this->t('Status Report'), Url::fromRoute('system.status'));
    $status_report = $status_report->toRenderable();
    global $_gitinfo_extra_folders_;
    if (isset($_gitinfo_extra_folders_)) {
      $disabled = TRUE;
      $explain_title = $this->t("(<strong>FIELD INPUT DISABLED</strong>)");
      $description = $this->t("Overridden by the global variable '\$_gitinfo_extra_folders_' in the 'settings.php' file with the values (comma seperated):<br /><em>$_gitinfo_extra_folders_</em>");
    }
    $config = $this->config('gitinfo.settings');
    $form['link'] = [
      '#type' => 'item',
      '#description' => $this->t("Configuration for the GIT Info section in the @status_report.", ['@status_report' => render($status_report)]),
    ];
    $form['connect'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Connect with remote repos'),
      '#description' => $this->t("<strong>Slows down report generation</strong> but it tells if your working directory is behind compared with the remote repo. In other words, if a 'git pull' is needed. Furthermore, it shows if your Git version is up to date."),
      '#default_value' => $config->get('connect'),
    ];
    $form['extra_folders'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Extra folders to monitor outside the current project root ') . $explain_title,
      '#description' => $description,
      '#default_value' => $config->get('extra_folders'),
      '#disabled' => $disabled,
    ];
    $form['aggregate_folders'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Aggregate items in ignored folders to show only an item count per directory'),
      '#description' => $this->t('To save vertical space in the report. One directory path per line. The <strong>relative path</strong> from the project root must be used.'),
      '#default_value' => $config->get('aggregate_folders'),
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $extras = array_map('trim', explode(PHP_EOL, $form_state->getValue('extra_folders')));
    foreach ($extras as $extra) {
      $exists = shell_exec('[ -d ' . escapeshellcmd('/' . trim($extra, " /\t\n\r")) . ' ] && echo "1"');
      if (!$exists) {
        $form_state->setErrorByName('extra_folders', $this->t('Directory %directory does not exist.', ['%directory' => '/' . trim($extra, " /\t\n\r")]));
      }
    }
    $aggregates = array_map('trim', explode(PHP_EOL, $form_state->getValue('aggregate_folders')));
    foreach ($aggregates as $aggregate) {
      if (!preg_match('/^[\w|\/]+$/', $aggregate)) {
        $form_state->setErrorByName('aggregate_folders', $this->t('Directory %directory is invalid.', ['%directory' => trim($aggregate, " /\t\n\r")]));
      }
    }
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $extra_folders = [];
    $extras = explode(PHP_EOL, $form_state->getValue('extra_folders'));
    foreach ($extras as $extra) {
      $extra_folders[] = '/' . trim($extra, " /\t\n\r");
    }
    $extra_folders = implode(PHP_EOL, $extra_folders);
    $aggregate_folders = [];
    $aggregates = explode(PHP_EOL, $form_state->getValue('aggregate_folders'));
    foreach ($aggregates as $aggregate) {
      $aggregate_folders[] = trim($aggregate, " /\t\n\r");
    }
    $aggregate_folders = implode(PHP_EOL, $aggregate_folders);
    $this->config('gitinfo.settings')
      ->set('connect', $form_state->getValue('connect'))
      ->set('extra_folders', $extra_folders)
      ->set('aggregate_folders', $aggregate_folders)
      ->save();
    parent::submitForm($form, $form_state);
  }

}
